@@ -35,6 +35,7 @@
 #endif
 
 #include <QCheckBox>
+#include <QCloseEvent>
 #include <QDesktopServices>
 #include <QFont>
 
@@ -168,6 +169,11 @@ void DatabaseOpenWidget::hideEvent(QHideEvent* event)
     }
 }
 
+bool DatabaseOpenWidget::unlockingDatabase()
+{
+    return m_unlockingDatabase;
+}
+
 void DatabaseOpenWidget::load(const QString& filename)
 {
     clearForms();
@@ -207,6 +213,7 @@ void DatabaseOpenWidget::clearForms()
     m_ui->editPassword->setShowPassword(false);
     m_ui->keyFileLineEdit->clear();
     m_ui->keyFileLineEdit->setShowPassword(false);
+    m_ui->keyFileLineEdit->setClearButtonEnabled(true);
     m_ui->challengeResponseCombo->clear();
     m_ui->centralStack->setCurrentIndex(0);
     m_db.reset();
@@ -334,7 +341,12 @@ QSharedPointer<CompositeKey> DatabaseOpenWidget::buildDatabaseKey()
 #ifdef Q_CC_MSVC
         if (!getWindowsHello()->getKey(m_filename, keyData)) {
             // Failed to retrieve Quick Unlock data
-            m_ui->messageWidget->showMessage(tr("Failed to authenticate with Windows Hello"), MessageWidget::Error);
+            auto error = getWindowsHello()->errorString();
+            if (!error.isEmpty()) {
+                m_ui->messageWidget->showMessage(tr("Failed to authenticate with Windows Hello: %1").arg(error),
+                                                 MessageWidget::Error);
+                resetQuickUnlock();
+            }
             return {};
         }
 #elif defined(Q_OS_MACOS)
@@ -436,12 +448,6 @@ void DatabaseOpenWidget::browseKeyFile()
     }
 }
 
-void DatabaseOpenWidget::clearKeyFileText()
-{
-    m_ui->keyFileLineEdit->clear();
-    m_ui->keyFileLineEdit->setShowPassword(false);
-}
-
 void DatabaseOpenWidget::pollHardwareKey()
 {
     if (m_pollingHardwareKey) {
@@ -522,6 +528,7 @@ void DatabaseOpenWidget::setUserInteractionLock(bool state)
         }
         m_ui->centralStack->setEnabled(true);
     }
+    m_unlockingDatabase = state;
 }
 
 bool DatabaseOpenWidget::isOnQuickUnlockScreen()
@@ -536,9 +543,6 @@ bool DatabaseOpenWidget::isOnQuickUnlockScreen()
  */
 void DatabaseOpenWidget::resetQuickUnlock()
 {
-    if (!isQuickUnlockAvailable()) {
-        return;
-    }
 #if defined(Q_CC_MSVC)
     getWindowsHello()->reset(m_filename);
 #elif defined(Q_OS_MACOS)
