@@ -1,6 +1,6 @@
 /*
  *  Copyright (C) 2010 Felix Geyer <debfx@fobos.de>
- *  Copyright (C) 2017 KeePassXC Team <team@keepassxc.org>
+ *  Copyright (C) 2020 KeePassXC Team <team@keepassxc.org>
  *
  *  This program is free software: you can redistribute it and/or modify
  *  it under the terms of the GNU General Public License as published by
@@ -20,13 +20,14 @@
 #define KEEPASSX_MAINWINDOW_H
 
 #include <QActionGroup>
+#include <QLabel>
 #include <QMainWindow>
+#include <QProgressBar>
 #include <QSystemTrayIcon>
 
-#include "core/ScreenLockListener.h"
 #include "core/SignalMultiplexer.h"
-#include "gui/Application.h"
 #include "gui/DatabaseWidget.h"
+#include "gui/osutils/ScreenLockListener.h"
 
 namespace Ui
 {
@@ -35,6 +36,7 @@ namespace Ui
 
 class InactivityTimer;
 class SearchWidget;
+class MainWindowEventFilter;
 
 class MainWindow : public QMainWindow
 {
@@ -49,6 +51,8 @@ public:
     ~MainWindow();
 
     QList<DatabaseWidget*> getOpenDatabases();
+    void restoreConfigState();
+    void setAllowScreenCapture(bool state);
 
     enum StackedWidgetIndex
     {
@@ -58,9 +62,16 @@ public:
         PasswordGeneratorScreen = 3
     };
 
+signals:
+    void databaseUnlocked(DatabaseWidget* dbWidget);
+    void databaseLocked(DatabaseWidget* dbWidget);
+    void activeDatabaseChanged(DatabaseWidget* dbWidget);
+
 public slots:
     void openDatabase(const QString& filePath, const QString& password = {}, const QString& keyfile = {});
     void appExit();
+    bool isHardwareKeySupported();
+    bool refreshHardwareKeys();
     void displayGlobalMessage(const QString& text,
                               MessageWidget::MessageType type,
                               bool showClosebutton = true,
@@ -80,6 +91,7 @@ public slots:
     void bringToFront();
     void closeAllDatabases();
     void lockAllDatabases();
+    void closeModalWindow();
     void displayDesktopNotification(const QString& msg, QString title = "", int msTimeoutHint = 10000);
     void restartApp(const QString& message);
 
@@ -119,13 +131,13 @@ private slots:
     void clearLastDatabases();
     void updateLastDatabasesMenu();
     void updateCopyAttributesMenu();
+    void updateSetTagsMenu();
     void showEntryContextMenu(const QPoint& globalPos);
     void showGroupContextMenu(const QPoint& globalPos);
     void applySettingsChanges();
     void trayIconTriggered(QSystemTrayIcon::ActivationReason reason);
     void processTrayIconTrigger();
     void lockDatabasesAfterInactivity();
-    void forgetTouchIDAfterInactivity();
     void handleScreenLock();
     void showErrorMessage(const QString& message);
     void selectNextDatabaseTab();
@@ -134,9 +146,10 @@ private slots:
     void obtainContextFocusLock();
     void releaseContextFocusLock();
     void agentEnabled(bool enabled);
-
-private slots:
     void updateTrayIcon();
+    void updateProgressBar(int percentage, QString message);
+    void updateEntryCountLabel();
+    void focusSearchWidget();
 
 private:
     static void setShortcut(QAction* action, QKeySequence::StandardKey standard, int fallback = 0);
@@ -162,12 +175,16 @@ private:
     QPointer<QMenu> m_entryNewContextMenu;
     QPointer<QActionGroup> m_lastDatabasesActions;
     QPointer<QActionGroup> m_copyAdditionalAttributeActions;
+    QPointer<QActionGroup> m_setTagsMenuActions;
     QPointer<InactivityTimer> m_inactivityTimer;
     QPointer<InactivityTimer> m_touchIDinactivityTimer;
     int m_countDefaultAttributes;
     QPointer<QSystemTrayIcon> m_trayIcon;
     QPointer<ScreenLockListener> m_screenLockListener;
     QPointer<SearchWidget> m_searchWidget;
+    QPointer<QProgressBar> m_progressBar;
+    QPointer<QLabel> m_progressBarLabel;
+    QPointer<QLabel> m_statusBarLabel;
 
     Q_DISABLE_COPY(MainWindow)
 
@@ -176,13 +193,27 @@ private:
     bool m_restartRequested = false;
     bool m_contextMenuFocusLock = false;
     bool m_showToolbarSeparator = false;
+    bool m_allowScreenCapture = false;
     qint64 m_lastFocusOutTime = 0;
     qint64 m_lastShowTime = 0;
     QTimer m_updateCheckTimer;
     QTimer m_trayIconTriggerTimer;
     QSystemTrayIcon::ActivationReason m_trayIconTriggerReason;
+
+    friend class MainWindowEventFilter;
 };
 
+#if QT_VERSION >= QT_VERSION_CHECK(5, 15, 0)
+class MainWindowEventFilter : public QObject
+{
+    Q_OBJECT
+
+public:
+    explicit MainWindowEventFilter(QObject* parent);
+    bool eventFilter(QObject* watched, QEvent* event) override;
+};
+#endif
+
 /**
  * Return instance of MainWindow created on app load
  * non-gui instances will return nullptr
